"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixture = void 0;
const tslib_1 = require("tslib");
const fast_element_1 = require("@microsoft/fast-element");
const fast_foundation_1 = require("@microsoft/fast-foundation");
function findElement(view) {
    let current = view.firstChild;
    while (current !== null && current.nodeType !== 1) {
        current = current.nextSibling;
    }
    return current;
}
function isElementRegistry(obj) {
    return typeof obj.register === 'function';
}
/**
 * Creates a test fixture suitable for testing custom elements, templates, and bindings.
 * @param templateNameOrRegistry - An HTML template or single element name to create the fixture for.
 * @param options - Enables customizing fixture creation behavior.
 * @returns A promise of the test fixture {@link Fixture}.
 *
 * @public
 * @remarks
 * Yields control to the caller one Microtask later, in order to
 * ensure that the DOM has settled. This has changed in the latest version of FAST!
 */
function fixture(templateNameOrRegistry, options = {}) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const document = options.document || globalThis.document;
        const parent = options.parent || document.createElement('div');
        const source = options.source || {};
        const context = options.context || fast_element_1.defaultExecutionContext;
        if (typeof templateNameOrRegistry === 'string') {
            const html = `<${templateNameOrRegistry}></${templateNameOrRegistry}>`;
            templateNameOrRegistry = new fast_element_1.ViewTemplate(html, []);
        }
        else if (isElementRegistry(templateNameOrRegistry)) {
            templateNameOrRegistry = [templateNameOrRegistry];
        }
        if (Array.isArray(templateNameOrRegistry)) {
            const first = templateNameOrRegistry[0];
            const ds = options.designSystem || fast_foundation_1.DesignSystem.getOrCreate(parent);
            let prefix = '';
            ds.register(templateNameOrRegistry, {
                register(container, dsContext) {
                    prefix = dsContext.elementPrefix;
                },
            });
            const elementName = `${prefix}-${first.definition.baseName}`;
            const html = `<${elementName}></${elementName}>`;
            templateNameOrRegistry = new fast_element_1.ViewTemplate(html, []);
        }
        const view = templateNameOrRegistry.create();
        const element = findElement(view);
        let isConnected = false;
        view.bind(source, context);
        view.appendTo(parent);
        customElements.upgrade(parent);
        // Hook into the Microtask Queue to ensure the DOM is settled
        // before yielding control to the caller.
        yield Promise.resolve();
        const connect = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (isConnected) {
                return;
            }
            isConnected = true;
            document.body.appendChild(parent);
            yield Promise.resolve();
        });
        const disconnect = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!isConnected) {
                return;
            }
            isConnected = false;
            document.body.removeChild(parent);
            yield Promise.resolve();
        });
        return {
            document,
            template: templateNameOrRegistry,
            view,
            parent,
            element,
            connect,
            disconnect,
        };
    });
}
exports.fixture = fixture;
