"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.timeout = exports.test = exports.suite = exports.assert = void 0;
const tslib_1 = require("tslib");
const uvu_1 = require("uvu");
Object.defineProperty(exports, "suite", { enumerable: true, get: function () { return uvu_1.suite; } });
Object.defineProperty(exports, "test", { enumerable: true, get: function () { return uvu_1.test; } });
const assert = tslib_1.__importStar(require("uvu/assert"));
exports.assert = assert;
/**
 * Timeout utility.
 *
 * @example A promise that never ends.
 * ```ts
 * test(
 *   'should fail',
 *   timeout(async () => {
 *     await new Promise(() => {});
 *     assert.ok(true);
 *   }, 5_000),
 * );
 * ```
 *
 * @example A slow API.
 * ```ts
 * test(
 *   'should fail',
 *   timeout(async () => {
 *     const slowMockAPI = delayedResolve({ foo: 'bar' }, 4_000);
 *     await slowMockAPI();
 *     assert.unreachable('should have timed out');
 *   }, 2_000),
 * );
 * ```
 *
 * @param callback - The async test function.
 * @param duration - An optional duration in milliseconds. Defaults to 5_000.
 * @public
 */
const timeout = (callback, duration = 5000) => {
    return (context) => {
        let timer;
        return Promise.race([
            callback(context),
            new Promise((resolve, reject) => {
                timer = setTimeout(() => reject(new Error('timed out')), duration);
            }),
        ]).finally(() => {
            clearTimeout(timer);
        });
    };
};
exports.timeout = timeout;
