import type { FoundationElement, FoundationElementDefinition, FoundationElementRegistry, Registration } from '@microsoft/fast-foundation';
import { Container, DesignSystem } from '@microsoft/fast-foundation';
import { Fixture } from '../component';
import { assert, uvu } from '../uvu';
export { default as sinon } from 'sinon';
/**
 * @public
 */
export type RunCases = (fn: (...args: unknown[]) => unknown, cases: [unknown[], unknown, string?][], assertion?: (...args: unknown[]) => void) => void;
/**
 * Assert a set of test cases.
 * @param fn - The function to test.
 * @param cases - An array of test cases to run against the function.
 * @param assertion - Optional uvu assertion method, defaults to assert.equal.
 * @public
 * @remarks
 * This is available as part of the suite context, but can also be run standalone.
 */
export declare function runCases(fn: (...args: unknown[]) => unknown, cases: [unknown[], unknown, string?][], assertion?: (...args: unknown[]) => void): void;
/**
 * @public
 */
export type ElementGetter = () => FoundationElementRegistry<FoundationElementDefinition, FoundationElement>;
/**
 * Component suite context interface
 * @public
 */
export interface ComponentContext<TElement = HTMLElement> extends Pick<Fixture<TElement>, 'element' | 'disconnect'> {
    designSystem: DesignSystem;
    container: Container;
    runCases: RunCases;
}
/**
 * Logic suite context interface
 * @public
 */
export interface LogicContext {
    /**
     * @privateRemarks
     * Not hardening the typing on this to RunCases until get a feel for usages.
     */
    runCases?(fn: (value: any) => any, cases: [any, any, assert.Message?][]): void;
}
/**
 * Defines the Generic type for a test Suite `T` the callback function
 * required to assert on type `T`.
 * @typeParam T - the top level class we are testing
 * @public
 */
export type SuiteCallback<T> = uvu.Callback<ComponentContext<T>>;
/**
 * Create component test suite.
 *
 * @example
 * Simple suite using the tag name of the component.
 * ```ts
 * import { createComponentSuite } from '@genesislcap/foundation-testing';
 * import { Component } from './component';
 * Component; // < As we're using tag name in the Suite, we hold a reference to avoid tree shaking.
 * const Suite = createComponentSuite<Component>('Component', 'my-component');
 * // test cases...
 * Suite.run();
 * ```
 *
 * @example
 * Mocking a DI dependency for a composable component.
 * ```ts
 * const connectMock = new ConnectMock();
 * const mocks = [Registration.instance(Connect, connectMock)];
 * const Suite = createComponentSuite<ConnectionIndicator>('ConnectionIndicator Component', () => connectionIndicator(), null, mocks);
 * // test cases...
 * Suite.run();
 * ```
 *
 * @example
 * An element will be required to test anything that directly or in-directly makes use of the DI container, for example,
 * a service that can be injected into components, or has its own injected dependencies.
 * ```ts
 * import { Service } from './service';
 * @customElement({
 *   name: 'test-element',
 *   template: html`<slot></slot>`,
 * })
 * class TestElement extends FASTElement {}
 * const mocks = [...];
 * const Suite = createComponentSuite<TestElement>('Service', 'test-element', null, mocks);
 * // test cases...
 * Suite.run();
 * ```
 *
 * Importing the service should invoke the Service's DI registration, so in your test cases you can simply query the
 * container to get a reference to your service.
 * ```ts
 * Suite('Service.x does something expected', async ({ container }) => {
 *   const myService = container.get(Service);
 *   // assert
 * });
 * ```
 *
 * You can optionally add the service to the test element for lookup convenience, but this is not required.
 * ```ts
 * class TestElement extends FASTElement {
 *   @Service service: Service;
 * }
 * Suite('Element has service injected', async ({ element }) => {
 *   assert.ok(element.service);
 * });
 * ```
 *
 * @typeParam TElement - The element interface.
 * @param title - Title of the test suite
 * @param elementNameOrGetter - Element tag name or getter which is used to create the element within the fixture
 * @param context - Optional component context {@link ComponentContext}
 * @param registrations - Optional array of DI container registrations
 * @returns The test suite
 * @public
 * @remarks
 * Used to test function output given certain input arguments.
 */
export declare function createComponentSuite<TElement = HTMLElement>(title: string, elementNameOrGetter: string | ElementGetter, context?: ComponentContext<TElement>, registrations?: Registration<any>[]): uvu.Test<ComponentContext<TElement>>;
/**
 * Create logic test suite.
 *
 * @example
 * ```ts
 * import { createLogicSuite } from '@genesislcap/foundation-testing';
 * import { myFunction } from './logic';
 * const Suite = createLogicSuite('myFunction');
 * Suite('myFunction should provide expected results', ({ runCases }) => {
 *   runCases(myFunction, [
 *     [['1'], true],
 *     [[123], true],
 *     [['60%'], true],
 *     [['$60'], false],
 *     [['1.1'], false],
 *     [[''], false],
 *     [[true], false],
 *     [[null], false],
 *     [[undefined], false],
 *   ]);
 * });
 * Suite.run();
 * ```
 *
 * @typeParam TContext - The context interface.
 * @param title - Title of the test suite
 * @param context - Optional context which extends {@link LogicContext}
 * @returns The test suite
 * @public
 * @remarks
 * Used to test function output given certain input arguments.
 */
export declare function createLogicSuite<TContext = LogicContext>(title: string, context?: TContext): uvu.Test<TContext>;
//# sourceMappingURL=suite.d.ts.map