import sinon from 'sinon';
/**
 * Filters out key/value pairs from an object where they value does not match a condition
 * @public
 */
export type FilterConditionally<Source, Condition> = Pick<Source, {
    [K in keyof Source]: Source[K] extends Condition ? K : never;
}[keyof Source]>;
/**
 * Filters out all key/value pairs which are not functions, and omits the constructor
 * @public
 */
export type SpiedFunctions<T> = Omit<FilterConditionally<T, (...args: any[]) => any>, 'constructor'>;
/**
 * Crates an object with a key value pair for each spied function
 * where the key is the function name and the value is the sinon wrapped
 * spy function
 * @public
 * @typeParam T - object with keys where values are functions
 */
export type SinonWrapper<T extends {
    [K in keyof T]: (...args: any) => any;
}> = {
    [K in keyof T]: sinon.SinonSpy<Parameters<T[K]>, ReturnType<T[K]>>;
};
/**
 * Type can be used  to inform Typescript the object decorated with {@link testSpy} has function
 * spies which can be interacted upon their normal function reference
 * @public
 */
export type WithTestHarness<T extends {
    [k in keyof T]: any;
}> = T & SinonWrapper<SpiedFunctions<T>>;
/**
 * Decorator: Used on a test harness class based on a `FoundationElement` to give it extra functionality
 * which can be used during testing. *Important* this is to be used on a parent element compared to
 * the element under test.
 *
 *
 * @remarks
 * Access the test functionality using the type with {@link WithTestHarness}
 *
 * Reset after each test with {@link resetTestHarness}
 *
 * @example
 *
 * // Testing the first function call
 * async (\{ element \}) =\> \{
 *
    assert.ok(
      element.layout.addItemFromChild.calledWith(\{
        type: 'component',
        componentType: 'test',
        title: `Item test`,
        reorderEnabled: true,
        isClosable: false,
        size: undefined,
      \})
    );
 *
 *   // Reset the tester at the end of every test even if you don't assert on any of it!
*   resetTestHarness(element.layout);
 * \}
 *
 * @public
 */
export declare function testSpy(constructor: Function): void;
/**
 * Resets the history of the spied functions on the objects so previous running tests
 * don't affect the current test.
 * Must be called at the end of every test even if not asserting on the spies.
 * @param wrapper - `WithTestHarness<T>` object to reset
 * @public
 */
export declare function resetTestHarness<T>(wrapper: WithTestHarness<T>): void;
/**
 * Restores the spied functions back to the original functions without the spies.
 * @param wrapper - `WithTestHarness<T>` object to restore
 * @public
 */
export declare function restoreTestHarness<T>(wrapper: WithTestHarness<T>): void;
//# sourceMappingURL=harness.d.ts.map