import sinon from 'sinon';
/**
 * Predicate function used to check whether the object's value is a function
 * we want to spy on. We want to spy on all functions, except the constructor
 * @internal
 */
const shouldSpy = (proto) => (k) => typeof proto[k] === 'function' && k !== 'constructor';
/**
 * Decorator: Used on a test harness class based on a `FoundationElement` to give it extra functionality
 * which can be used during testing. *Important* this is to be used on a parent element compared to
 * the element under test.
 *
 *
 * @remarks
 * Access the test functionality using the type with {@link WithTestHarness}
 *
 * Reset after each test with {@link resetTestHarness}
 *
 * @example
 *
 * // Testing the first function call
 * async (\{ element \}) =\> \{
 *
    assert.ok(
      element.layout.addItemFromChild.calledWith(\{
        type: 'component',
        componentType: 'test',
        title: `Item test`,
        reorderEnabled: true,
        isClosable: false,
        size: undefined,
      \})
    );
 *
 *   // Reset the tester at the end of every test even if you don't assert on any of it!
*   resetTestHarness(element.layout);
 * \}
 *
 * @public
 */
export function testSpy(constructor) {
    constructor['_test'] = sinon.createSandbox();
    Object.getOwnPropertyNames(constructor.prototype)
        .filter(shouldSpy(constructor.prototype))
        .forEach((p) => constructor['_test'].spy(constructor.prototype, p));
}
/**
 * Resets the history of the spied functions on the objects so previous running tests
 * don't affect the current test.
 * Must be called at the end of every test even if not asserting on the spies.
 * @param wrapper - `WithTestHarness<T>` object to reset
 * @public
 */
export function resetTestHarness(wrapper) {
    wrapper.constructor['_test'].resetHistory();
}
/**
 * Restores the spied functions back to the original functions without the spies.
 * @param wrapper - `WithTestHarness<T>` object to restore
 * @public
 */
export function restoreTestHarness(wrapper) {
    wrapper.constructor['_test'].restore();
}
